import argparse
import hashlib
import json

import yaml
from mergedeep import Strategy, merge

from configutils.parser import ConfigFileParser, InlineJsonParser, StrParser


def merge_parameters(a, b):
    return merge(a, b, strategy=Strategy.REPLACE)


def get_config():
    parser = argparse.ArgumentParser(allow_abbrev=False)

    parser_actions = {
        parser.add_argument(
            "-c",
            help="Provide config file in yaml or json.",
            action="append",
        ).dest: ConfigFileParser(),
        parser.add_argument(
            "-j",
            help="Inline json.",
            action="append",
        ).dest: InlineJsonParser(),
        parser.add_argument(
            "-p",
            help="Provide extra parameters on the form key1.key2[key3]=value.",
            action="append",
        ).dest: StrParser(),
    }
    arguments, unknown_args = parser.parse_known_args()
    args = vars(arguments)
    current_parameters = {}

    for key in args:
        parser_action = parser_actions[key]
        if args[key] is not None:
            for value in args[key]:
                merge_parameters(current_parameters, parser_action.do(value))

    return current_parameters


def get_config_hash():
    return get_dict_hash(get_config())


def get_dict_hash(dictionary):
    dictionary_str = json.dumps(dictionary, sort_keys=True)
    dictionary_hash = hashlib.md5(dictionary_str.encode("utf-8")).hexdigest()
    return dictionary_hash


def save_config(pre_path):
    with open(f"{pre_path}{get_config_hash()}.yaml", "w") as f:
        yaml.dump(get_config(), f)



import abc
import json
import os
import re
from json import JSONDecodeError

import yaml


def value_parser(value):
    special_key = "SPECIAL_KEY"
    if re.match("^[-+]?[0-9]*\\.?[0-9]+(e[-+]?[0-9]+)?$", value) is None:
        return str(value)
    else:
        return yaml.safe_load(f"{special_key}: {value}")[special_key]


class Parser(abc.ABC, metaclass=abc.ABCMeta):
    def do(self, parameter_value: str):
        return self._do(parameter_value)

    @abc.abstractmethod
    def _do(self, parameter_value: str) -> dict:
        raise NotImplementedError


class ConfigFileParser(Parser):
    def __init__(self):
        self.file_parsers = {
            ".yaml": yaml.full_load,
            ".yml": yaml.full_load,
            ".json": json.load,
        }

    def _do(self, parameter_value: str) -> dict:
        path = parameter_value
        extension = os.path.splitext(path)[1]
        with open(path, "r") as f:
            return self.file_parsers[extension](f)


class StrParser(Parser):
    @staticmethod
    def key_value_to_dict(key, value):
        if not value:
            return None
        splits = key.split(".", maxsplit=1)
        current_key = splits[0]
        next_key = splits[1] if len(splits) > 1 else None
        if next_key is None:
            dictionary = {current_key: value}
        else:
            dictionary = {
                current_key: StrParser.key_value_to_dict(next_key, value)
            }
        return dictionary

    def _do(self, parameter_value: str) -> dict:
        splits = str(parameter_value).split("=", maxsplit=1)
        key, value = splits[0], value_parser(splits[1])
        return StrParser.key_value_to_dict(key, value)


class InlineJsonParser(Parser):
    def _do(self, parameter_value: str):
        try:
            return json.loads(str(parameter_value))
        except JSONDecodeError as e:
            print(e)
            return None



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['configutils']

package_data = \
{'': ['*']}

install_requires = \
['PyYAML>=6.0,<7.0', 'mergedeep>=1.3.4,<2.0.0']

setup_kwargs = {
    'name': 'serval-config-utils',
    'version': '0.1.1',
    'description': 'SerVal-Config-Utils automatically parse configurations from multiple sources into a single python dictionary.',
    'long_description': '# SerVal-Config-Utils\n\n [![License: MIT](https://img.shields.io/badge/License-MIT-yellow.svg)](https://opensource.org/licenses/MIT)\n [![test](https://github.com/UL-SnT-Serval/python-config-parser/actions/workflows/build.yaml/badge.svg)](https://github.com/UL-SnT-Serval/python-config-parser/actions/workflows/build.yaml)\n\n[![Bugs](https://sonarcloud.io/api/project_badges/measure?project=python-config-parser&metric=bugs)](https://sonarcloud.io/summary/new_code?id=python-config-parser)\n[![Code Smells](https://sonarcloud.io/api/project_badges/measure?project=python-config-parser&metric=code_smells)](https://sonarcloud.io/summary/new_code?id=python-config-parser)\n[![Coverage](https://sonarcloud.io/api/project_badges/measure?project=python-config-parser&metric=coverage)](https://sonarcloud.io/summary/new_code?id=python-config-parser)\n[![Duplicated Lines (%)](https://sonarcloud.io/api/project_badges/measure?project=python-config-parser&metric=duplicated_lines_density)](https://sonarcloud.io/summary/new_code?id=python-config-parser)\n[![Vulnerabilities](https://sonarcloud.io/api/project_badges/measure?project=python-config-parser&metric=vulnerabilities)](https://sonarcloud.io/summary/new_code?id=python-config-parser)\n\n## Description\n\nSerVal-Config-Utils automatically parse configurations from multiple sources into a single python dictionary.\n\n## Installation\n\n### Dependencies\n\nserval-config-utils requires:\n- PyYAML (^6.0)\n- mergedeep (^1.3.4)\n\n### User installation\nThe easiest way to install serval-config-parser is using `pip`\n\n```\npip install -U serval-config-utils\n```\n\n## Usage\n\nIn the python main file, use:\n```\nimport configutils\nconfig = configutils.get_config()\n```\n\nCall the main file with parameters\n```\npython main.py  -c ./path/to/config.yaml \\\n                -c ./path/to/config.json \\\n                -p my.nested.parameter=value \\\n                -j {"json_formatted":{"nested_parameter":"value"}} \\\n```\nAccess the merged config in the `config` dictionary.\n### Example\nThis simple examples merge the config from `examples/basic_config.yaml` and `examples/basic_config.json` and prints it in the standard output.\n```\npython examples/basic_example.py -c examples/basic_config.yaml -c examples/basic_config.json\n```\n\n## Development\n\nWe welcome new contributors of all experience levels.\nUse pre-commit to ensure your code follows standards.\n\n### Source code\n```\ngit clone https://github.com/serval-uni-lu/python-config-parser\n```\n\n<!-- ### Test\nAfter installation, you can launch the test suite from outside the source directory (you will need to have pytest >= 5.0.1 installed):\n```\npytest configutils\n``` -->\n',
    'author': 'Thibault Simonetto',
    'author_email': 'thibault.simonetto@uni.lu',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.6.2,<4.0.0',
}


setup(**setup_kwargs)
